import { Component, Injectable, Input, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subject as Subject$1 } from 'rxjs/Subject';

class LoadingBarService {
    constructor() {
        this.progress$ = new Subject$1();
        this._pending = new Subject$1();
        this._pendingRequests = 0;
        this._value = 0;
    }
    /**
     * @return {?}
     */
    get pending() {
        console.warn(`LoadingBarService: 'pending' is deprecated and will removed in the next version use 'progress$' instead which return a value between 0 and 100.`);
        return this._pending;
    }
    /**
     * @return {?}
     */
    start() {
        this._pending.next(++this._pendingRequests);
        if (this._value === 0 || this._value === 100) {
            // Inserts the loading bar element into the dom, and sets it to 2%
            this.set(2);
        }
    }
    /**
     * @return {?}
     */
    complete() {
        if (this._pendingRequests === 0) {
            return;
        }
        this._pending.next(--this._pendingRequests);
        if (this._pendingRequests === 0 && this._value !== 100) {
            if (this._value > 0) {
                this.set(100);
                // Attempt to aggregate any start/complete calls within 500ms:
                setTimeout(() => {
                    if (this._value === 100) {
                        this.progress$.next(0);
                        this._value = 0;
                    }
                }, 500);
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._pending.complete();
        this.progress$.complete();
    }
    /**
     * Set the loading bar's width to a certain percent.
     *
     * @param {?} n any value between 0 and 100
     * @return {?}
     */
    set(n) {
        this._value = n;
        this.progress$.next(n);
        // increment loadingbar to give the illusion that there is always
        // progress but make sure to cancel the previous timeouts so we don't
        // have multiple incs running at the same time.
        clearTimeout(this._incTimeout);
        if (this._value < 100) {
            this._incTimeout = setTimeout(() => this.increment(), 250);
        }
    }
    /**
     * Increments the loading bar by a random amount
     * but slows down as it progresses
     * @return {?}
     */
    increment() {
        let /** @type {?} */ rnd = 0;
        const /** @type {?} */ stat = this._value;
        if (stat >= 0 && stat < 25) {
            // Start out between 3 - 6% increments
            rnd = (Math.random() * (5 - 3 + 1) + 3);
        }
        else if (stat >= 25 && stat < 65) {
            // increment between 0 - 3%
            rnd = (Math.random() * 3);
        }
        else if (stat >= 65 && stat < 90) {
            // increment between 0 - 2%
            rnd = (Math.random() * 2);
        }
        else if (stat >= 90 && stat < 99) {
            // finally, increment it .5 %
            rnd = 0.5;
        }
        else {
            // after 99%, don't increment:
            rnd = 0;
        }
        this.set(this._value + rnd);
    }
}
LoadingBarService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
LoadingBarService.ctorParameters = () => [];

class LoadingBarComponent {
    /**
     * @param {?} loader
     */
    constructor(loader) {
        this.loader = loader;
        this.includeSpinner = true;
        this.progress$ = this.loader.progress$;
    }
}
LoadingBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-loading-bar, ngx-loading-bar',
                template: `
    <ng-container *ngIf="(progress$|async) as value">
      <div id="loading-bar-spinner" *ngIf="includeSpinner"><div class="spinner-icon"></div></div>
      <div id="loading-bar"><div class="bar" [style.width]="value + '%'"><div class="peg"></div></div></div>
    </ng-container>
  `,
            },] },
];
/**
 * @nocollapse
 */
LoadingBarComponent.ctorParameters = () => [
    { type: LoadingBarService, },
];
LoadingBarComponent.propDecorators = {
    'includeSpinner': [{ type: Input },],
};

class LoadingBarModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: LoadingBarModule,
            providers: [LoadingBarService],
        };
    }
}
LoadingBarModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [LoadingBarComponent],
                exports: [LoadingBarComponent],
            },] },
];
/**
 * @nocollapse
 */
LoadingBarModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { LoadingBarModule, LoadingBarComponent, LoadingBarService };
//# sourceMappingURL=core.js.map
